import asyncio
import json
import logging

from aiohttp import ClientResponse, ClientSession
from gestion_sports.gestion_sports_payload_builder import \
    GestionSportsPayloadBuilder
from models import BookingFilter, User

LOGGER = logging.getLogger(__name__)
HEADERS = {
    "Connection": "keep-alive",
    "Accept-Language": "en-US,en;q=0.5",
    "Accept-Encoding": "gzip, deflate, br",
    "DNT": "1",
    "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
    "Accept": "application/json, text/javascript, */*; q=0.01",
    "X-Requested-With": "XMLHttpRequest",
    "Sec-Fetch-Dest": "empty",
    "Sec-Fetch-Mode": "cors",
    "Sec-Fetch-Site": "same-origin",
}


class GestionSportsConnector:

    def __init__(self, session: ClientSession, url: str):
        LOGGER.info("Initializing connection to GestionSports API")
        self.url = url
        self.session = session
        self.payload_builder = GestionSportsPayloadBuilder()

    @property
    def connection_url(self) -> str:
        return f"{self.url}/connexion.php?"

    @property
    def login_url(self) -> str:
        return f"{self.url}/connexion.php?"

    @property
    def booking_url(self) -> str:
        return f"{self.url}/membre/reservation.html?"

    async def connect(self) -> ClientResponse:
        LOGGER.info("Connecting to GestionSports API")
        async with self.session.get(self.connection_url) as response:
            await response.text()
            return response

    async def login(self, user: User) -> ClientResponse:
        payload = (
            self.payload_builder.login(user.login)
            .password(user.password)
            .club_id(user.club_id)
            .build_login_payload()
        )

        async with self.session.post(
            self.login_url, data=payload, headers=HEADERS
        ) as response:
            await response.text()
            return response

    async def book(self, booking_filter: BookingFilter) -> int | None:
        bookings = await asyncio.gather(
            *[
                self.book_one_court(booking_filter, court_id)
                for court_id in booking_filter.court_ids
            ],
            return_exceptions=True,
        )

        for court, is_booked in bookings:
            if is_booked:
                return court
        return None

    async def book_one_court(
        self, booking_filter: BookingFilter, court_id: int
    ) -> tuple[int, bool]:
        date_format = "%d/%m/%Y"
        time_format = "%H:%M"
        payload = (
            self.payload_builder.date(booking_filter.date.date().strftime(date_format))
            .time(booking_filter.date.time().strftime(time_format))
            .sport_id(booking_filter.sport_id)
            .court_id(court_id)
            .build_booking_payload()
        )
        return court_id, await self.is_court_booked(payload)

    async def is_court_booked(self, payload: str) -> bool:
        async with self.session.post(
            self.booking_url, data=payload, headers=HEADERS
        ) as response:
            return self.is_response_status_ok(await response.text())

    @staticmethod
    def is_response_status_ok(response: str) -> bool:
        formatted_result = response.removeprefix('"').removesuffix('"')
        result_json = json.loads(formatted_result)
        return result_json["status"] == "ok"
