import pytest
from aiohttp import ClientSession
from yarl import URL

from resa_padel.gestion_sports.gestion_sports_connector import \
    GestionSportsConnector
from tests.fixtures import (a_booking_failure_response, a_booking_filter,
                            a_booking_payload, a_booking_success_response,
                            a_connector, a_user)

gestion_sports_url = "https://toulousepadelclub.gestion-sports.com"


@pytest.mark.asyncio
async def test_should_connect_to_gestion_sports_website():
    async with ClientSession() as session:
        cookies = session.cookie_jar.filter_cookies(URL(gestion_sports_url))
        assert cookies.get("PHPSESSID") is None
        gs_connector = GestionSportsConnector(session, gestion_sports_url)

        response = await gs_connector.connect()

        assert response.status == 200
        assert response.request_info.method == "GET"
        assert response.content_type == "text/html"
        assert response.request_info.url == URL(gestion_sports_url + "/connexion.php")
        assert response.charset == "UTF-8"

        cookies = session.cookie_jar.filter_cookies(URL(gestion_sports_url))
        assert cookies.get("PHPSESSID") is not None


@pytest.mark.asyncio
async def test_should_login_to_gestion_sports_website(a_user):
    async with ClientSession() as session:
        gs_connector = GestionSportsConnector(session, gestion_sports_url)
        await gs_connector.connect()

        response = await gs_connector.login(a_user)

        assert response.status == 200
        assert response.request_info.url == URL(gestion_sports_url + "/connexion.php")
        assert response.request_info.method == "POST"

        cookies = session.cookie_jar.filter_cookies(URL(gestion_sports_url))
        assert cookies.get("COOK_ID_CLUB").value is not None
        assert cookies.get("COOK_ID_USER").value is not None
        assert cookies.get("PHPSESSID") is not None


@pytest.mark.asyncio
async def test_booking_url_should_be_reachable(a_user, a_booking_filter):
    async with ClientSession() as session:
        gs_connector = GestionSportsConnector(session, gestion_sports_url)
        await gs_connector.connect()
        await gs_connector.login(a_user)

        court_booked = await gs_connector.book(a_booking_filter)
        # At 18:00 no chance to get a booking, any day of the week
        assert court_booked is None


@pytest.mark.asyncio
async def test_should_book_a_court_from_gestion_sports(
    aioresponses,
    a_booking_filter,
    a_booking_success_response,
    a_booking_failure_response,
):
    booking_url = URL(gestion_sports_url + "/membre/reservation.html?")

    # first booking request will fail
    aioresponses.post(URL(booking_url), status=200, body=a_booking_failure_response)
    # first booking request will succeed
    aioresponses.post(URL(booking_url), status=200, body=a_booking_success_response)
    # first booking request will fail
    aioresponses.post(URL(booking_url), status=200, body=a_booking_failure_response)

    async with ClientSession() as session:
        gs_connector = GestionSportsConnector(session, gestion_sports_url)
        court_booked = await gs_connector.book(a_booking_filter)

        # the second element of the list is the booked court
        assert court_booked == a_booking_filter.court_ids[1]


def test_response_status_should_be_ok(a_booking_success_response):
    is_booked = GestionSportsConnector.is_response_status_ok(a_booking_success_response)
    assert is_booked


def test_response_status_should_be_not_ok(aioresponses, a_booking_failure_response):
    is_booked = GestionSportsConnector.is_response_status_ok(a_booking_failure_response)
    assert not is_booked


@pytest.mark.asyncio
async def test_court_should_not_be_booked(
    aioresponses, a_connector, a_booking_payload, a_booking_failure_response
):
    aioresponses.post(
        URL(a_connector.booking_url), status=200, body=a_booking_failure_response
    )
    is_booked = await a_connector.is_court_booked(a_booking_payload)
    assert not is_booked


@pytest.mark.asyncio
async def test_court_should_be_booked(
    aioresponses, a_connector, a_booking_payload, a_booking_success_response
):
    aioresponses.post(
        URL(a_connector.booking_url), status=200, body=a_booking_success_response
    )
    is_booked = await a_connector.is_court_booked(a_booking_payload)
    assert is_booked
